"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var assets_loader_helper_1 = require("./assets-loader.helper");
/**
 * Robin Coma Delperier
 * Licensed under the Apache-2.0 License
 * https://github.com/PlaceMe-SAS/single-spa-angular-cli/blob/master/LICENSE
 *
 * modified by Phodal HUANG
 *
 */
var loadScriptTag = function (src) {
    return function () {
        return new Promise(function (resolve, reject) {
            var script = assets_loader_helper_1.default.createScriptTag(src);
            script.onload = function () {
                resolve();
            };
            script.onerror = function (err) {
                reject(err);
            };
            document.head.appendChild(script);
        });
    };
};
var loadLinkTag = function (url) {
    return function () {
        return new Promise(function (resolve, reject) {
            var link = assets_loader_helper_1.default.createLinkTag(url);
            link.onload = function () {
                resolve();
            };
            link.onerror = function (err) {
                reject(err);
            };
            document.head.appendChild(link);
        });
    };
};
function loadAllAssets(opts) {
    return new Promise(function (resolve, reject) {
        var scriptsPromise = opts.scripts.reduce(function (prev, fileName) {
            return prev.then(loadScriptTag(opts.baseScriptUrl + "/" + fileName));
        }, Promise.resolve(undefined));
        var stylesPromise = opts.styles.reduce(function (prev, fileName) {
            return prev.then(loadLinkTag(opts.baseScriptUrl + "/" + fileName));
        }, Promise.resolve(undefined));
        Promise.all([scriptsPromise, stylesPromise]).then(resolve, reject);
    });
}
function unloadTag(opts, scriptName) {
    return function () {
        return new Promise(function (resolve, reject) {
            var tag = document.getElementById(assets_loader_helper_1.default.hashCode(opts.baseScriptUrl + "/" + scriptName));
            if (tag) {
                document.head.removeChild(tag);
            }
            resolve();
        });
    };
}
var MooaLoaderHelper = {
    loadAllAssets: loadAllAssets,
    unloadTag: unloadTag
};
exports.default = MooaLoaderHelper;
//# sourceMappingURL=mooa-loader.helper.js.map