"use strict";
/**
 * Robin Coma Delperier
 * Licensed under the Apache-2.0 License
 * https://github.com/PlaceMe-SAS/single-spa-angular-cli/blob/master/LICENSE
 *
 * modified by Phodal HUANG
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
var mooa_loader_helper_1 = require("../helper/mooa-loader.helper");
var app_helper_1 = require("../helper/app.helper");
function bootstrap(opts) {
    if (!window['mooa']) {
        window.mooa = {};
    }
    window.mooa.isSingleSpa = true;
    window.mooa.name = opts.name;
    return new Promise(function (resolve, reject) {
        mooa_loader_helper_1.default.loadAllAssets(opts.appConfig).then(resolve, reject);
    });
}
function load(app) {
    return Promise.resolve();
}
function mount(app, props) {
    return new Promise(function (resolve, reject) {
        app_helper_1.getAppElement(app.appConfig);
        if (window.mooa[app.name]) {
            window.mooa[app.name].mount(props);
            resolve();
        }
        else {
            console.error("Cannot mount " + app.name + " because that is not bootstraped");
            reject();
        }
    });
}
function unmount(app, props) {
    var unloadApplication = props.unloadApplication, getAppNames = props.getAppNames;
    return new Promise(function (resolve, reject) {
        if (window.mooa[app.name]) {
            window.mooa[app.name].unmount();
            app_helper_1.removeAppElement(app.appConfig);
            if (getAppNames().indexOf(app.name) !== -1) {
                unloadApplication(app.name, { waitForUnmount: true });
                resolve();
            }
            else {
                reject("Cannot unmount " + app.name + " because that " + app.name + " is not part of the decalred applications : " + getAppNames());
            }
        }
        else {
            reject("Cannot unmount " + app.name + " because that is not bootstraped");
        }
    });
}
function unload(app) {
    return new Promise(function (resolve, reject) {
        app.appConfig.scripts
            .concat(app.appConfig.styles)
            .reduce(function (prev, scriptName) {
            return prev.then(mooa_loader_helper_1.default.unloadTag(app.appConfig, scriptName));
        }, Promise.resolve({}));
        resolve();
    });
}
function mooaLoader(opts) {
    return {
        bootstrap: bootstrap.bind(null, opts),
        load: load.bind(null, opts),
        mount: mount.bind(null, opts),
        unload: unload.bind(null, opts),
        unmount: unmount.bind(null, opts)
    };
}
exports.default = mooaLoader;
//# sourceMappingURL=mooa.loader.js.map